#include <stdlib.h>
#include <stdio.h>
#include "aswm/mapper/mapper.h"
#include "aswm/log/log.h"

/**
 * Returns the free tile built in the split process.
 */
void h_split(Display* display, Window current_node, Window window_to_map);

void aswm_map_new_window(Display* display, Window current_node, Window window_to_map) {
	h_split(display, current_node, window_to_map);
}

void h_split(Display* display, Window current_tile, Window window_to_map) {
	// Current:
	// +--------------+
	// | current tile |
	// | +--------+   |
	// | | child  |   |
	// | +--------+   |
	// +--------------+
	// Target:
	// +---------------+
	// | current tile  |
	// | +-----------+ |
	// | | child     | |
	// | +-----------+ |
	// | +-----------+ |
	// | | free tile | |
	// | +-----------+ |
	// +---------------+

	XWindowAttributes current_tile_attributes;
	XGetWindowAttributes(display, current_tile, &current_tile_attributes);

	XReparentWindow(display, window_to_map, current_tile, 0, 0);

	Window _root_window; // unused
	Window _parent_window; // unused
	Window* children;
	unsigned int children_count;

	XQueryTree(display, current_tile, &_root_window, &_parent_window, &children, &children_count);

	for(auto i = 0; i < children_count; i++) {
		XMoveResizeWindow(display, children[i],
				0, i * current_tile_attributes.height / children_count,
				current_tile_attributes.width,
				current_tile_attributes.height / children_count);
	}
	XFree(children);
	XMapWindow(display, window_to_map);
}

void resize_children_to_parent(Display* display, Window parent) {
	Window _root_window; // unused
	Window _grand_parent; // unused
	Window* children;
	unsigned int children_count;
	XQueryTree(
			display, parent,
			&_root_window, &_grand_parent,
			&children, &children_count);
	printf("  Resize children of %lu:\n", parent);
	for(auto i = 0; i < children_count; i++)
		printf("%lu ", children[i]);
	printf("\n");

	// Resize new children of the parent tile
	XWindowAttributes parent_attributes;
	XGetWindowAttributes(display, parent, &parent_attributes);

	for(auto i = 0; i < children_count; i++) {
		// TODO: currently only hsplit supported
		XMoveResizeWindow(display, children[i],
				0,
				i * parent_attributes.height / children_count,
				parent_attributes.width,
				parent_attributes.height / children_count);
	}
	XFree(children);
}

void aswm_unmap_window(Display* display, Window parent, Window window) {
	// +-------------------------------+
	// | root                          |
	// |+-------------+ +-------------+|
	// || parent      | |             ||
	// || +--------+  | |             ||
	// || | win 1  |  | |             ||
	// || +--------+  | |             ||
	// || +--------+  | |             ||
	// || | del win|  | |    win 3    ||
	// || +--------+  | |             ||
	// || +--------+  | |             ||
	// || | win 2  |  | |             ||
	// || +--------+  | |             ||
	// |+-------------+ +-------------+|
	// +-------------------------------+

	Window _root_window; // unused
	Window _grand_parent; // unused
	Window* children;
	unsigned int children_count;
	XQueryTree(
			display, parent,
			&_root_window, &_grand_parent,
			&children, &children_count);
	printf("  Current children of %lu:\n", parent);
	for(auto i = 0; i < children_count; i++)
		printf("%lu ", children[i]);
	printf("\n");
	for(auto i = 0; i < children_count; i++) {
		if(children[i] == window)
			// If the unmap request does not come from a destroy window, we
			// still want to remove the window from the ASWM hierarchy. It is
			// the responsability of the client to decide to remap it latter or
			// not.
			XReparentWindow(display, window, XDefaultRootWindow(display), 0, 0);
	}

	resize_children_to_parent(display, parent);
}
